#pragma once

namespace limax {

	enum
	{
		SOURCE_LIMAX = 0,
		SOURCE_PLAT = 1,
		SOURCE_ENDPOINT = 2,

		SUCCEED = 0,
		AUANY_PLAT_HAS_ERROR = 1,
		SWITCHER_AUANY_TIMEOUT = 11,
		SWITCHER_SESSION_LOGINED = 12,
		SWITCHER_SEND_DISPATCH_EXCEPTION = 13,
		SWITCHER_SEND_TO_ENDPOINT_EXCEPTION = 14,
		SWITCHER_DHGROUP_NOTSUPPRTED = 15,
		SWITCHER_LOST_PROVIDER = 16,
		SWITCHER_PROVIDER_UNBIND = 17,
		SWITCHER_WRONG_PROVIDER = 18,
		AUANY_UNKNOWN_PLAT = 1001,
		AUANY_BAD_TOKEN = 1002,
		AUANY_AUTHENTICATE_TIMEOUT = 1003,
		AUANY_AUTHENTICATE_FAIL = 1004,
		AUANY_CALL_PROCEDURE_FAILED = 1005,
		AUANY_CHECK_LOGIN_IP_FAILED = 1006,
		AUANY_CHECK_PROVIDER_KEY_UNKNOWN_PVID = 1007,
		AUANY_CHECK_PROVIDER_KEY_BAD_KEY = 1008,
		AUANY_EXPERIENCE_UPGRADE_CLIENT_TIMEOUT = 1011,
		AUANY_EXPERIENCE_UPGRADE_MAMAGER_CLOSE = 1012,
		AUANY_EXPERIENCE_UPGRADE_ENGINE_CLOSE = 1013,
		AUANY_EXPERIENCE_UPGRADE_BAD_ARGS = 1021,
		AUANY_EXPERIENCE_UPGRADE_SRC_ACCOUNT_NOT_EXPE = 1022,
		AUANY_EXPERIENCE_UPGRADE_SRC_ACCOUNT_NOT_EXISTING = 1023,
		AUANY_EXPERIENCE_UPGRADE_DST_ACCOUNT_EXISTING = 1024,
		AUANY_EXPERIENCE_UPGRADE_DST_ACCOUNT_IS_EXPE = 1025,
		ENDPOINT_PING_TIMEOUT = 2001,
		PROVIDER_DUPLICATE_ID = 3002,
		PROVIDER_LOST_SESSION_TRANSPORT = 3003,
		PROVIDER_UNSUPPORTED_VARINAT = 3004,
		PROVIDER_NOT_ALLOW_VARINAT = 3005,
		PROVIDER_UNSUPPORTED_SCRIPT = 3006,
		PROVIDER_NOT_ALLOW_SCRIPT = 3007,
		PROVIDER_KICK_SESSION = 3008,

		SYSTEM_SUCCEED = SUCCEED,
		SYSTEM_ENDPOINT_NO_AVAILABLE_PROVIDER = 2,
		SYSTEM_ENDPOINT_RECV_UNKNOWN_PROTOCOL = 3,
		SYSTEM_VIEW_MARSHAL_EXCEPTION = 4,
		SYSTEM_VIEW_LOST_INSTANCE = 5,
		SYSTEM_VIEW_LOST_FIELD = 6,
		SYSTEM_VIEW_BAD_PROTOCOL_DATA = 7,
		SYSTEM_PARSE_VARIANT_DEFINES_EXCEPTION = 8,
	};

	struct LIMAX_DLL_EXPORT_API EndpointConfig;
	struct LIMAX_DLL_EXPORT_API EndpointManager
	{
		EndpointManager();
		virtual ~EndpointManager();
		virtual Transport* getTransport() = 0;
		virtual void close() = 0;
		virtual int64_t getSessionId() const = 0;
		virtual int64_t getAccountFlags() const = 0;
		virtual ViewContext* getViewContext(int16_t pvid, ViewContext::Type type) const = 0;
	};

	struct LIMAX_DLL_EXPORT_API Transport
	{
		virtual void sendProtocol(const limax::Protocol&) = 0;
		Transport();
		virtual ~Transport();
		virtual EndpointManager* getManager() = 0;
		virtual const struct sockaddr_in& getPeerAddress() const = 0;
		virtual const struct sockaddr_in& getLocalAddress() const = 0;
		virtual void* getSessionObject() = 0;
		virtual void setSessionObject(void *) = 0;
	};

	struct LIMAX_DLL_EXPORT_API EndpointConfig
	{
		EndpointConfig();
		virtual ~EndpointConfig();
		virtual int getDHGroup() const = 0;
		virtual const std::string& getServerIP() const = 0;
		virtual int getServerPort() const = 0;
		virtual const std::string& getUserName() const = 0;
		virtual const std::string& getToken() const = 0;
		virtual const std::string& getPlatFlag() const = 0;
		virtual bool isPingServerOnly() const = 0;
		virtual std::shared_ptr<State> getEndpointState() const = 0;
		virtual Executor getExecutor() const = 0;
		virtual const std::vector<std::shared_ptr<View::ViewCreatorManager>>& getStaticViewCreatorManagers() const = 0;
		virtual const std::vector<int16_t>& getVariantProviderIds() const = 0;
		virtual std::shared_ptr<ScriptEngineHandle> getScriptEngineHandle() const = 0;
	};

	struct LIMAX_DLL_EXPORT_API EndpointListener
	{
		EndpointListener();
		virtual ~EndpointListener();
		virtual void onManagerInitialized(EndpointManager*, EndpointConfig*) = 0;
		virtual void onTransportAdded(Transport*) = 0;
		virtual void onTransportRemoved(Transport*) = 0;
		virtual void onAbort(Transport*) = 0;
		virtual void onManagerUninitialized(EndpointManager*) = 0;
		virtual void onSocketConnected() = 0;
		virtual void onKeyExchangeDone() = 0;
		virtual void onKeepAlived(int ping) = 0;
		virtual void onErrorOccured(int errorsource, int errorvalue, const std::string& info) = 0;
		virtual void destroy() = 0;
	};


	struct LIMAX_DLL_EXPORT_API EndpointConfigBuilder
	{
		EndpointConfigBuilder();
		virtual ~EndpointConfigBuilder();
		virtual std::shared_ptr<EndpointConfigBuilder> endpointState(std::initializer_list<std::shared_ptr<State>>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> endpointState(std::vector<std::shared_ptr<State>>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> staticViewCreatorManagers(std::initializer_list<std::shared_ptr<View::ViewCreatorManager>>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> staticViewCreatorManagers(std::vector<std::shared_ptr<View::ViewCreatorManager>>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> variantProviderIds(std::initializer_list<int16_t>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> variantProviderIds(std::vector<int16_t>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> scriptEngineHandle(std::shared_ptr<ScriptEngineHandle>) = 0;
		virtual std::shared_ptr<EndpointConfigBuilder> executor(Executor) = 0;
		virtual std::shared_ptr<EndpointConfig> build() const = 0;
	};

	struct LIMAX_DLL_EXPORT_API Endpoint
	{
		static void openEngine();
		static void closeEngine(Runnable);
		static std::shared_ptr<EndpointConfigBuilder> createLoginConfigBuilder(const std::string& serverip, int serverport, const std::string& username, const std::string& token, const std::string& platflag);
		static std::shared_ptr<EndpointConfigBuilder> createPingOnlyConfigBuilder(const std::string& serverip, int serverport);
		static void start(std::shared_ptr<EndpointConfig>, EndpointListener* handler);
		static EndpointManager* getDefaultEndpointManager();
	};

	class LIMAX_DLL_EXPORT_API Expansion
	{
		EndpointManager* manager;
	public:
		Expansion();
		Expansion(EndpointManager*);
		~Expansion();
	public:
		Expansion(const Expansion&);
		Expansion(const Expansion&&);
		Expansion& operator=(const Expansion&);
		Expansion& operator=(const Expansion&&);
	public:
		void experienceUpgrade(const std::string& username, const std::string& token, const std::string& platflag, std::function<void(int, int)> onresult);
	};

} // namespace limax {
