package limax.auany;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.net.SocketAddress;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import limax.auany.auanyviews.ExperienceUpgrade;
import limax.auany.plats.Experience;
import limax.auany.switcherauany.SessionAuthByToken;
import limax.defines.ErrorCodes;
import limax.defines.ErrorSource;
import limax.defines.SessionFlags;
import limax.provider.ProcedureHelper;
import limax.switcherauany.AuanyAuthArg;
import limax.switcherauany.AuanyAuthRes;
import limax.util.Trace;
import limax.zdb.Procedure;

final class PlatManager {

	private final static Map<String, PlatProcess> plats = new ConcurrentHashMap<>();

	private PlatManager() {
	}

	static void insertPlatProcess(String type, PlatProcess pp) {
		if (null != plats.put(type.toLowerCase(), pp))
			throw new RuntimeException("duplicate play process type " + type);
	}

	static byte[] onHttpServerCallBack(String path, String query) {
		return null;
	}

	private static boolean checkClientAddress(SocketAddress peeraddress,
			SocketAddress reportaddress, int type) {
		return true;
	}

	private static void response(SessionAuthByToken rpc) {
		try {
			rpc.response();
		} catch (Exception e) {
			if (Trace.isErrorEnabled())
				Trace.error(rpc, e);
		}
	}

	static void process(SessionAuthByToken rpc) {
		AuanyAuthArg arg = rpc.getArgument();
		final AuanyAuthRes result = rpc.getResult();
		result.errorSource = ErrorSource.LIMAX;

		SocketAddress peeraddress;
		SocketAddress reportaddress;
		try (final ObjectInputStream ois = new ObjectInputStream(
				new ByteArrayInputStream(arg.clientaddress.array(), 0,
						arg.clientaddress.size()))) {
			peeraddress = (SocketAddress) ois.readObject();
			reportaddress = (SocketAddress) ois.readObject();
		} catch (Exception e) {
			peeraddress = reportaddress = new SocketAddress() {
				private static final long serialVersionUID = -210184168558204443L;
			};
		}
		if (!checkClientAddress(peeraddress, reportaddress, arg.sockettype)) {
			result.errorCode = ErrorCodes.AUANY_CHECK_LOGIN_IP_FAILED;
			response(rpc);
			return;
		}
		final String platstring = arg.platflag.toLowerCase();
		final PlatProcess pp = plats.get(platstring);
		if (null == pp) {
			result.errorCode = ErrorCodes.AUANY_UNKNOWN_PLAT;
			response(rpc);
			return;
		}
		pp.checkUserToken(arg.username, arg.token, (userid, errorSource,
				errorCode) -> {
			if (ErrorCodes.SUCCEED != errorCode
					|| ErrorSource.LIMAX != errorSource) {
				result.errorSource = errorSource;
				result.errorCode = errorCode;
				response(rpc);
				return;
			}
			result.errorCode = ErrorCodes.AUANY_CALL_PROCEDURE_FAILED;
			Procedure.execute(
					new GetSessionProcedure(platstring,
							pp instanceof Experience, arg.username, userid,
							arg.pvids.keySet()), (p, r) -> {
						if (r.isSuccess()) {
							result.sessionid = p.getSessionId();
							result.flags = p.getFlags();
							result.permissions.addAll(p.getPermission());
							result.errorSource = ErrorSource.LIMAX;
							result.errorCode = ErrorCodes.SUCCEED;
						}
						response(rpc);
					});
		});
	}

	static void onExperienceUpgrade(ExperienceUpgrade view, long sessionid,
			String username, String token, String platflag) {
		final PlatProcess pp = plats.get(platflag.toLowerCase());
		if (null == pp) {
			view.setErrorcodes(new limax.auanyviews.ErrorCodes(
					ErrorSource.LIMAX, ErrorCodes.AUANY_UNKNOWN_PLAT));
			view.destroyInstance();
			return;
		}
		if (pp instanceof Experience) {
			view.setErrorcodes(new limax.auanyviews.ErrorCodes(
					ErrorSource.LIMAX,
					ErrorCodes.AUANY_EXPERIENCE_UPGRADE_DST_ACCOUNT_IS_EXPE));
			view.destroyInstance();
			return;
		}
		try {
			pp.checkUserToken(
					username,
					token,
					(userid, errorSource, errorCode) -> {
						if (ErrorCodes.SUCCEED != errorCode
								|| ErrorSource.LIMAX != errorSource) {
							view.setErrorcodes(new limax.auanyviews.ErrorCodes(
									errorSource, errorCode));
							view.destroyInstance();
							return;
						}
						Procedure.execute(() -> {
							cbean.SessionKey key = table.Sessions
									.select(sessionid);
							if (null == key) {
								view._setErrorcodes(new limax.auanyviews.ErrorCodes(
										errorSource,
										ErrorCodes.AUANY_EXPERIENCE_UPGRADE_SRC_ACCOUNT_NOT_EXISTING));
								view.destroyInstance();
								return false;
							}
							xbean.SessionInfo info = table.Sessioninfos
									.select(key);
							if (SessionFlags.FLAG_EXPERIENCE_ACCOUNT != (info
									.getFlags() & SessionFlags.FLAG_EXPERIENCE_ACCOUNT)) {
								view._setErrorcodes(new limax.auanyviews.ErrorCodes(
										errorSource,
										ErrorCodes.AUANY_EXPERIENCE_UPGRADE_SRC_ACCOUNT_NOT_EXPE));
								view.destroyInstance();
								return false;
							}

							final cbean.SessionKey newkey = new cbean.SessionKey(
									platflag, userid);
							if (null != table.Sessioninfos.select(newkey)) {
								view._setErrorcodes(new limax.auanyviews.ErrorCodes(
										errorSource,
										ErrorCodes.AUANY_EXPERIENCE_UPGRADE_DST_ACCOUNT_EXISTING));
								view.destroyInstance();
								return false;
							}

							table.Sessions.delete(sessionid);
							table.Sessions.insert(sessionid, newkey);
							xbean.SessionInfo newinfo = table.Sessioninfos
									.insert(newkey);
							newinfo.setFlags(info.getFlags()
									& ~SessionFlags.FLAG_EXPERIENCE_ACCOUNT);
							newinfo.setSessionid(sessionid);
							newinfo.setUsername(username);
							table.Sessioninfos.delete(key);
							view.setErrorcodes(new limax.auanyviews.ErrorCodes(
									ErrorSource.LIMAX, ErrorCodes.SUCCEED));
							ProcedureHelper.executeWhileCommit(() -> view
									.destroyInstance());
							return true;
						});
					});
		} catch (Exception e) {
			if (Trace.isErrorEnabled())
				Trace.error("PlatManager onExperienceUpgrade", e);
		}

	}

}
