package limax.auany;

import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.ReflectionException;

import limax.auany.switcherauany.CheckProviderKey;
import limax.defines.ErrorCodes;
import limax.util.MBeans;
import limax.util.Trace;

final class ProviderKeyStore {

	private final static Map<Integer, String> keymap = new ConcurrentHashMap<>();

	static void insertKey(int pvid, String key) {
		keymap.put(pvid, key);
	}

	static int checkKey(int pvid, String key) {
		final String s = keymap.get(pvid);
		if (null == s)
			return ErrorCodes.AUANY_CHECK_PROVIDER_KEY_UNKNOWN_PVID;
		if (s.equals(key))
			return ErrorCodes.SUCCEED;
		else
			return ErrorCodes.AUANY_CHECK_PROVIDER_KEY_BAD_KEY;
	}

	static void process(CheckProviderKey rpc) {
		final limax.switcherauany.CheckProviderKeyArg arg = rpc.getArgument();
		rpc.getResult().errorcodes = checkKey(arg.pvid, arg.pvkey);
		if (Trace.isInfoEnabled())
			Trace.info("CheckProviderKey pvid = " + arg.pvid + " errorcodes = "
					+ rpc.getResult().errorcodes);
	}

	static {
		MBeans.register(MBeans.root(), new MBeanOperators(),
				"limax.auany:type=ProviderKeyStore");
	}

	private final static class MBeanOperators implements DynamicMBean {

		@Override
		public Object getAttribute(String attribute)
				throws AttributeNotFoundException, MBeanException,
				ReflectionException {
			switch (attribute) {
			case "keys": {
				return keymap;
			}
			default:
				break;
			}
			return null;
		}

		@Override
		public void setAttribute(Attribute attribute)
				throws AttributeNotFoundException,
				InvalidAttributeValueException, MBeanException,
				ReflectionException {
		}

		@Override
		public AttributeList getAttributes(String[] attributes) {
			return new AttributeList(
					Arrays.asList(new Attribute("keys", keymap)));
		}

		@Override
		public AttributeList setAttributes(AttributeList attributes) {
			return null;
		}

		@Override
		public Object invoke(String actionName, Object[] params,
				String[] signature) throws MBeanException, ReflectionException {
			switch (actionName) {
			case "providerid": {
				insertKey((short) params[0], params[1].toString());
				return 0;
			}
			default:
				break;
			}
			return null;
		}

		@Override
		public MBeanInfo getMBeanInfo() {
			MBeanAttributeInfo[] attrinfos = { new MBeanAttributeInfo("keys",
					Map.class.getName(), "keys", true, false, false) };
			MBeanOperationInfo[] operinfos = { new MBeanOperationInfo(
					"setOrInsertKey",
					"update the old key value, or insert a new key", Arrays
							.asList(new MBeanParameterInfo("providerid",
									short.class.getName(), "provider id"),
									new MBeanParameterInfo("key", String.class
											.getName(), "provider key"))
							.toArray(new MBeanParameterInfo[0]),
					String.class.getName(), MBeanOperationInfo.ACTION) };
			return new MBeanInfo(ProviderKeyStore.class.getName(),
					"provider keys", attrinfos, null, operinfos, null);
		}

	}
}
