package limax.auany;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.concurrent.ExecutorService;

import limax.util.ConcurrentEnvironment;
import limax.util.ElementHelper;
import limax.util.Trace;
import limax.xmlconfig.ConfigParser;
import limax.xmlconfig.Service;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.sun.net.httpserver.HttpServer;

public class XmlConfig {
	public interface DataMXBean {
		public String getHttpServerIP();

		public int getHttpServerPort();
	}

	public static final class LoadConfig implements ConfigParser {
		@Override
		public void parse(Element self) throws Exception {
			final ElementHelper eh = new ElementHelper(self);

			final String httpserverip = eh.getString("http.server.ip");
			final int httpserverport = eh.getInt("http.server.port", 0);

			if (!httpserverip.isEmpty() && httpserverport > 0) {
				Service.addRunAfterEngineStartTask(() -> {
					try {
						final InetSocketAddress sa = new InetSocketAddress(
								httpserverip, httpserverport);
						final HttpServer server = HttpServer.create(sa, 0);
						final ExecutorService executor = ConcurrentEnvironment
								.getInstance().newThreadPool(
										"limax.auany.httpserver", 0);
						server.setExecutor(executor);
						server.createContext("/", new HttpServerHandler());
						server.start();
						Service.addRunBeforeEngineStopTask(() -> server.stop(0));
					} catch (IOException e) {
						if (Trace.isErrorEnabled())
							Trace.error("start http ( httpserverip = "
									+ httpserverip + " httpserverport = "
									+ httpserverport + ")", e);
					}
				});
				Service.JMXRegister(new DataMXBean() {
					@Override
					public String getHttpServerIP() {
						return httpserverip;
					}

					@Override
					public int getHttpServerPort() {
						return httpserverport;
					}
				}, "limax.auany:type=XmlConfig,name=httpserver");
			}

			Service.JMXRegister(new MBeanOperators(),
					"limax.auany:type=XmlConfig,name=userinfo");

			{
				final NodeList plats = self.getElementsByTagName("plat");
				final int count = plats.getLength();
				for (int i = 0; i < count; i++)
					parsePlatElement((Element) plats.item(i));
			}

			{
				final NodeList providers = self
						.getElementsByTagName("provider");
				final int count = providers.getLength();
				for (int i = 0; i < count; i++)
					parseProviderElement((Element) providers.item(i));
			}

		}

		private void parsePlatElement(Element ele)
				throws ClassNotFoundException, InstantiationException,
				IllegalAccessException {

			final String classname = ele.getAttribute("className");
			final String platname = ele.getAttribute("name").toUpperCase();
			if (null == classname || classname.isEmpty())
				return;
			if (Trace.isDebugEnabled())
				Trace.debug("Config.load plat classname = " + classname
						+ " platname = " + platname);
			final Class<?> cls = Class.forName(classname);
			final PlatProcess process = (PlatProcess) cls.newInstance();
			process.init(ele);
			PlatManager.insertPlatProcess(platname, process);
		}

		private void parseProviderElement(Element ele) {
			final ElementHelper eh = new ElementHelper(ele);
			final int pvid = eh.getInt("id");
			final String key = eh.getString("key");
			ProviderKeyStore.insertKey(pvid, key);
		}
	}
}
