package limax.endpoint;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

import limax.defines.ErrorCodes;
import limax.defines.ErrorSource;
import limax.endpoint.Expansion.ExperienceUpgradeResult;
import limax.endpoint.auanyviews.ExperienceUpgrade;
import limax.endpoint.auanyviews.Service;
import limax.net.Engine;
import limax.util.Trace;

class ExpansionImpl {

	static Expansion createInstance(final EndpointManager manager) {
		return new Expansion() {

			@Override
			public void experienceUpgrade(String username, String token,
					String platflag, ExperienceUpgradeResult onresult)
					throws Exception {
				startExperienceUpgrade(new ExperienceUpgradeParam(manager,
						username, token, platflag, onresult));
			}

		};
	}

	private static class ExperienceUpgradeParam {

		public ExperienceUpgradeParam(EndpointManager manager, String username,
				String token, String platflag, ExperienceUpgradeResult onresult) {
			this.manager = manager;
			this.username = username;
			this.token = token;
			this.platflag = platflag;
			this.onresult = onresult;
		}

		final EndpointManager manager;
		final String username;
		final String token;
		final String platflag;
		final ExperienceUpgradeResult onresult;

		ScheduledFuture<?> timeoutFuture;
	}

	private static final Map<Long, ExperienceUpgradeParam> experienceUpgradeRequestMap = new ConcurrentHashMap<Long, ExperienceUpgradeParam>();
	private static final AtomicLong experienceUpgradeSnGenerator = new AtomicLong();

	private static void startExperienceUpgrade(
			final ExperienceUpgradeParam param) throws Exception {
		final long sn = experienceUpgradeSnGenerator.getAndIncrement();
		experienceUpgradeRequestMap.put(sn, param);
		param.timeoutFuture = Engine.getProtocolScheduler().schedule(
				new Runnable() {
					@Override
					public void run() {
						experienceUpgradeRequestMap.remove(sn);
						param.onresult.onExperienceUpgradeResult(
								ErrorSource.LIMAX,
								ErrorCodes.AUANY_EXPERIENCE_UPGRADE_CLIENT_TIMEOUT);
					}
				}, 1, TimeUnit.MINUTES);
		Service.getInstance(param.manager).ExperienceUpgrade(sn);
	}

	static void onTempViewOpen(final ExperienceUpgrade view) {
		view.registerListener("sn", new ViewChangedListener() {
			@Override
			public void onViewChanged(ViewChangedEvent e) {
				ExperienceUpgradeParam param = experienceUpgradeRequestMap
						.get(e.getValue());
				if (null != param) {
					try {
						view.Upgrade(param.username, param.token,
								param.platflag);
					} catch (Exception e1) {
						if (Trace.isErrorEnabled())
							Trace.error("ExperienceUpgrade Upgrade", e1);
					}
				}
			}
		});

		view.registerListener("errorcodes", new ViewChangedListener() {
			@Override
			public void onViewChanged(ViewChangedEvent e) {
				final limax.auanyviews.ErrorCodes errorcodes = (limax.auanyviews.ErrorCodes) e
						.getValue();
				view.visitSn(new ViewVisitor<Long>() {
					@Override
					public void accept(Long value) {
						ExperienceUpgradeParam param = experienceUpgradeRequestMap
								.remove(value);
						if (null != param && param.timeoutFuture.cancel(false))
							param.onresult.onExperienceUpgradeResult(
									errorcodes.errorSource,
									errorcodes.errorCode);
					}
				});
			}
		});
	}

	static void onTempViewClose(ExperienceUpgrade view) {
		view.visitSn(new ViewVisitor<Long>() {
			@Override
			public void accept(Long value) {
				ExperienceUpgradeParam param = experienceUpgradeRequestMap
						.remove(value);
				if (null != param && param.timeoutFuture.cancel(false))
					param.onresult.onExperienceUpgradeResult(ErrorSource.LIMAX,
							ErrorCodes.AUANY_EXPERIENCE_UPGRADE_MAMAGER_CLOSE);
			}
		});
	}

	static void cleanup() {
		for (ExperienceUpgradeParam param : experienceUpgradeRequestMap
				.values()) {
			if (param.timeoutFuture.cancel(false))
				param.onresult.onExperienceUpgradeResult(ErrorSource.LIMAX,
						ErrorCodes.AUANY_EXPERIENCE_UPGRADE_ENGINE_CLOSE);
		}
		experienceUpgradeRequestMap.clear();
	}
}
