package limax.provider;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import limax.codec.StringStream;
import limax.provider.providerendpoint.SyncViewToClients;
import limax.util.Resource;
import limax.util.Trace;

public abstract class GlobalView extends View {
	private final Map<String, Byte> map = new LinkedHashMap<>();

	protected GlobalView(ViewContext context, ViewStub stub, Resource resource,
			String[] varnames) {
		super(context, stub, resource, varnames.length);
		for (int i = 0; i < varnames.length; i++)
			map.put(varnames[i], (byte) i);
	}

	@Override
	public void hashSchedule(Runnable task) {
		getViewContext().hashSchedule(this, task);
	}

	public final void syncToClient(Collection<Long> sessionids, String varname) {
		syncToClient(sessionids, Arrays.asList(map.get(varname)));
	}

	public final void syncToClient(Collection<Long> sessionids) {
		syncToClient(sessionids, map.values());
	}

	public final void syncToClient(long sessionid, String varname) {
		syncToClient(Arrays.asList(sessionid), varname);
	}

	public final void syncToClient(long sessionid) {
		syncToClient(Arrays.asList(sessionid));
	}

	private void syncToClient(Collection<Long> sessionids,
			Collection<Byte> varindexes) {
		hashSchedule(() -> {
			if (isClosed())
				return;
			Collection<DataItem> items = getRecentDataItem(varindexes);
			if (items.isEmpty())
				return;
			final SyncViewToClients protocol = new SyncViewToClients();
			protocol.providerid = getProviderId();
			protocol.sessionids.addAll(sessionids);
			protocol.classindex = getClassIndex();
			protocol.instanceindex = 0;
			protocol.synctype = SyncViewToClients.DT_VIEW_DATA;
			items.forEach(i -> protocol.vardatas.add(i.data));
			StringStream sstream = prepareStringHeader(protocol);
			if (sstream != null) {
				items.forEach(i -> sstream.append(i.text));
				protocol.stringdata = sstream.toString(":P:");
			}
			try {
				getViewContext().syncViewToClients(protocol);
			} catch (Throwable e) {
				if (Trace.isErrorEnabled())
					Trace.error("syncToClient view = " + this, e);
			}
		});
	}

	@Override
	void close() {
		hashSchedule(() -> super.close());
	}
}
