package limax.provider;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Map;
import java.util.Set;

import limax.codec.Octets;
import limax.codec.StringStream;
import limax.provider.providerendpoint.SyncViewToClients;
import limax.util.Resource;

public abstract class SessionView extends AutoView {
	private final long sessionid;
	private Map<TemporaryView, Set<Byte>> subscribes;

	protected SessionView(ViewContext context, ViewStub stub,
			Resource resource, long sessionid, String[] touch) {
		super(context, stub, resource, touch);
		this.sessionid = sessionid;
	}

	public final long getSessionId() {
		return sessionid;
	}

	@Override
	public void hashSchedule(Runnable task) {
		getViewContext().hashSchedule(getSessionId(), task);
	}

	@Override
	Collection<Long> getSessionIds() {
		return Arrays.asList(sessionid);
	}

	@Override
	void assembleProtocolData(SyncViewToClients protocol, StringStream sstream) {
		protocol.stringdata = sstream.toString(":P:");
	}

	@Override
	byte getSyncDataType() {
		return SyncViewToClients.DT_VIEW_DATA;
	}

	Collection<DataItem> subscribe(TemporaryView tview,
			Collection<Byte> varindexes) {
		if (subscribes == null)
			subscribes = new IdentityHashMap<>();
		Set<Byte> set = subscribes.get(tview);
		if (set == null)
			subscribes.put(tview, set = new HashSet<>());
		set.addAll(varindexes);
		return getRecentDataItem(varindexes);
	}

	void unsubscribe(TemporaryView tview) {
		if (subscribes == null)
			return;
		subscribes.remove(tview);
		if (subscribes.isEmpty())
			subscribes = null;
	}

	@Override
	final DataItem syncToClient(byte varindex, Octets vardata, String strdata) {
		DataItem item = super.syncToClient(varindex, vardata, strdata);
		if (item != null && subscribes != null)
			subscribes
					.entrySet()
					.stream()
					.filter(e -> e.getValue().contains(varindex))
					.map(e -> e.getKey())
					.forEach(
							view -> view.onSessionViewUpdate(sessionid,
									getClassIndex(), item));
		return item;
	}

	@Override
	void close() {
		hashSchedule(() -> super.close());
	}
}
