package limax.xmlgen;

import limax.util.ElementHelper;

import org.w3c.dom.Element;

public final class Rpc extends Naming {
	private Rpc self;

	private String argument;
	private String result;
	private short typeHigh;
	private short typeLow;
	private int maxsize;
	private int timeout;

	private Bean implementBean;
	private Bean argumentBean;
	private Bean resultBean;

	public Rpc(Namespace namespace, Element self) throws Exception {
		super(namespace, self);
		initialize(self);
	}

	public Rpc(State state, Element self) throws Exception {
		super(state, self, "ref");
		new ElementHelper(self).warnUnused("ref");
	}

	void initialize(Element self) throws Exception {
		Variable.verifyName(this);
		this.self = this;
		implementBean = new Bean(this, self);
		ElementHelper eh = new ElementHelper(self);
		timeout = eh.getInt("timeout");
		argument = eh.getString("argument");
		result = eh.getString("result");
		typeLow = eh.getShort("type");
		maxsize = eh.getInt("maxsize");
		if (typeLow <= 0)
			throw new RuntimeException(getFullName() + ": type = "
					+ this.typeLow + " <= 0");
		if (maxsize < 0)
			throw new RuntimeException(getFullName() + ": maxsize = " + maxsize
					+ " < 0");
		eh.warnUnused("name");
	}

	@Override
	public boolean resolve() {
		if (!super.resolve())
			return false;
		if (self == null) {
			self = dereference(super.getName().split("\\."), getClass());
			return self != null;
		}
		this.typeHigh = getParent() instanceof Namespace ? ((Namespace) getParent())
				.getPvid() : (short) 0;
		if ((argumentBean = search(argument.split("\\."), Bean.class)) == null)
			return false;
		if ((resultBean = search(result.split("\\."), Bean.class)) == null)
			return false;
		Main.checkReserveType(getFullName(), typeHigh, typeLow);
		return true;
	}

	public Bean getArgumentBean() {
		return self.argumentBean;
	}

	public Bean getResultBean() {
		return self.resultBean;
	}

	public int getTimeout() {
		return self.timeout;
	}

	public int getType() {
		return self.typeHigh << 16 | self.typeLow;
	}

	public int getMaxsize() {
		return self.maxsize;
	}

	public short getTypeHigh() {
		return self.typeHigh;
	}

	public Bean getImplementBean() {
		return self.implementBean;
	}

	public String getLastName() {
		return self.getName();
	}

	public String getFirstName() {
		StringBuilder sb = new StringBuilder();
		for (Naming p = self.getParent(); p instanceof Namespace; p = p
				.getParent())
			sb.insert(0, p.getName()).insert(0, ".");
		return sb.deleteCharAt(0).toString();
	}

	public String getFullName() {
		return getFirstName() + "." + getLastName();
	}
}
