package limax.xmlgen.java;

import java.io.PrintStream;
import java.util.List;

import limax.xmlgen.Enum;
import limax.xmlgen.Variable;
import limax.xmlgen.Xbean;

class XbeanFormatter {
	public static void make(Xbean xbean) {
		try (PrintStream ps = Zdbgen.openXBeanFile(xbean.getName())) {
			new XbeanFormatter(xbean, ps).make();
		}
	}

	private final Xbean xbean;
	private final PrintStream ps;
	private final boolean isAny;
	private final List<Variable> variables;

	private XbeanFormatter(Xbean xbean, PrintStream ps) {
		this.xbean = xbean;
		this.ps = ps;
		this.isAny = xbean.isAny();
		this.variables = xbean.getVariables();
	}

	private void make() {
		String cls = xbean.getName();
		ps.println("package xbean;");
		ps.println();
		ps.println("import limax.codec.OctetsStream;");
		ps.println("import limax.codec.MarshalException;");
		if (xbean.attachment() != null)
			ps.println("import limax.codec.StringStream;");
		ps.println();
		ps.println("public final class "
				+ cls
				+ " extends limax.zdb.XBean"
				+ (xbean.attachment() == null ? ""
						: " implements limax.codec.StringMarshal") + " {");
		List<Enum> enums = xbean.getEnums();
		enums.forEach(e -> Declare.make(e, ps, "    "));
		if (!enums.isEmpty())
			ps.println();
		variables.forEach(var -> Declare.make(var, Declare.Type.PRIVATE, ps,
				"    "));
		ps.println();
		ps.println("    " + cls + "(limax.zdb.XBean _xp_, String _vn_) {");
		ps.println("        super(_xp_, _vn_);");
		for (Variable var : variables)
			Construct.make(var, ps, "        ");
		ps.println("	}");
		ps.println();
		ps.println("	public " + cls + "() {");
		ps.println("		this(null, null);");
		ps.println("	}");
		ps.println();

		ps.println("	public " + cls + "(" + cls + " _o_) {");
		ps.println("		this(_o_, null, null);");
		ps.println("	}");
		ps.println();
		ps.println("	" + cls + "(" + cls
				+ " _o_, limax.zdb.XBean _xp_, String _vn_) {");
		ps.println("		super(_xp_, _vn_);");
		Zdbgen.verify(ps, "		_o_.", cls, true);

		if (isAny) {
			ps.println("		throw new UnsupportedOperationException();");
			ps.println("	}");
			ps.println();
		} else {
			xbean.getVariables().forEach(
					var -> VarDeepCopy.make(var, ps, "        ", false));
			ps.println("	}");
			ps.println();
		}

		// marshal implement
		ps.println("	@Override");
		ps.println("	public final OctetsStream marshal(OctetsStream _os_) {");
		if (isAny)
			ps.println("		throw new UnsupportedOperationException();");
		else {
			Zdbgen.verify(ps, "        ", "marshal", true);
			for (Variable var : variables)
				Marshal.make(var, ps, "        ");

			ps.println("        return _os_;");
		}
		ps.println("    }");
		ps.println();

		if (xbean.attachment() != null) {
			ps.println("	@Override");
			ps.println("	public final StringStream marshal(StringStream _os_) {");
			if (isAny) {
				ps.println("		throw new UnsupportedOperationException();");
			} else {
				Zdbgen.verify(ps, "        ", "marshal", true);
				ps.println("		_os_.append(\"L\");");
				variables.forEach(var -> StringMarshal.make(var, ps, "		"));
				ps.println("		return _os_.append(\":\");");
			}
			ps.println("    }");
			ps.println();
		}

		ps.println("	@Override");
		ps.println("	public final OctetsStream unmarshal(OctetsStream _os_) throws MarshalException {");
		if (isAny)
			ps.println("		throw new UnsupportedOperationException();");
		else {
			Zdbgen.verify(ps, "		", "unmarshal", false);
			for (Variable var : variables) {
				Define.beginXbeanParent_VarName(var.getName());
				Unmarshal.make(var, ps, "		");
				Define.endXbeanParent_VarName();
			}
			ps.println("		return _os_;");
		}
		ps.println("	}");
		ps.println();

		// getter setter
		variables.forEach(var -> VarGetter.make(var, ps, "	"));
		variables.forEach(var -> VarSetter.make(var, ps, "	"));
		// equals
		ps.println("	@Override");
		ps.println("	public final boolean equals(Object _o1_) {");
		Zdbgen.verify(ps, "		", "equals", true);
		if (!variables.isEmpty()) {
			ps.println("		" + cls + " _o_ = null;");
			ps.println("		if ( _o1_ instanceof " + cls + " ) _o_ = (" + cls
					+ ")_o1_;");
			ps.println("		else return false;");

			for (Variable var : variables)
				Equals.make(var, ps, "		");
		}
		ps.println("		return true;");
		ps.println("	}");
		ps.println();
		Hashcode.make(xbean, ps, "	");
		Trace.make(xbean, ps, "	");
		if (xbean.isConstType())
			CompareTo.make(xbean, ps, "	");
		ps.println("}");
	}
}
