package limax.zdb;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class NoteMap<K, V> implements Note {
	private final Set<K> added = new HashSet<K>();
	private final Map<K, V> removed = new HashMap<K, V>();
	private final Map<K, V> replaced = new HashMap<K, V>();
	private List<V> changed;
	private Map<K, V> objref;
	private Map<K, V> changedmap;

	@SuppressWarnings("unchecked")
	final void setChanged(List<V> changed, Object objref) {
		this.changed = changed;
		this.objref = (Map<K, V>) objref;
		getChanged();
	}

	final void merge(Note note) {
		@SuppressWarnings("unchecked")
		NoteMap<K, V> another = (NoteMap<K, V>) note;
		for (K k : another.added)
			logPut(k, null, null);
		for (Map.Entry<K, V> e : another.removed.entrySet())
			logRemove(e.getKey(), e.getValue());
		for (Map.Entry<K, V> e : another.replaced.entrySet())
			logPut(e.getKey(), e.getValue(), e.getValue());
	}

	final Set<K> getAdded() {
		return added;
	}

	final Map<K, V> getReplaced() {
		return replaced;
	}

	public final Map<K, V> getRemoved() {
		return removed;
	}

	public final Map<K, V> getChanged() {
		if (changedmap != null)
			return changedmap;
		changedmap = new HashMap<K, V>(replaced);
		if (changed == null && added.isEmpty()) {
			changedmap.putAll(replaced);
			return changedmap;
		}
		added.forEach(key -> changedmap.put(key, objref.get(key)));
		if (changed != null) {
			Set<V> set = new HashSet<>(changed);
			for (Entry<K, V> e : objref.entrySet())
				if (set.contains(e.getValue()))
					changedmap.put(e.getKey(), e.getValue());
		}
		return changedmap;
	}

	final boolean isMapChanged() {
		return !added.isEmpty() || !removed.isEmpty() || !replaced.isEmpty();
	}

	final void clear() {
		added.clear();
		removed.clear();
		replaced.clear();
		objref = null;
		changed = null;
		changedmap = null;
	}

	final void logRemove(K key, V value) {
		if (added.remove(key))
			return;
		V v = replaced.remove(key);
		removed.put(key, v == null ? value : v);
	}

	final void logPut(K key, V origin, V value) {
		if (added.contains(key))
			return;
		V v = removed.remove(key);
		if (null != v) {
			replaced.put(key, v);
			return;
		}
		if (replaced.containsKey(key))
			return;
		if (null == origin)
			added.add(key);
		else
			replaced.put(key, origin);
	}

	@Override
	public String toString() {
		return "added=" + added + " replaced=" + replaced + " removed="
				+ removed + " changed=" + changed;
	}
}
