package limax.zdb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

class Tables {
	private final Map<String, AbstractTable> tables = new HashMap<String, AbstractTable>();
	private final ReentrantReadWriteLock flushLock = new ReentrantReadWriteLock();
	private LoggerEngine logger;

	private List<StorageInterface> storages = new ArrayList<StorageInterface>();
	private TableSys tableSys = new TableSys();

	final TableSys getTableSys() {
		return tableSys;
	}

	final void add(AbstractTable table) {
		if (null != tables.put(table.getName(), table))
			throw new XError("duplicate table name " + table.getName());
	}

	final void open(limax.xmlgen.Zdb meta) {
		if (null != logger)
			throw new XError("tables opened");
		switch (meta.getEngineType()) {
		case MYSQL:
			logger = new LoggerMysql(meta);
			break;
		case EDB:
			logger = new LoggerEdb(meta);
			break;
		}
		storages.add(tableSys.open(meta.getTable(tableSys.getName()), logger));
		Map<String, Integer> map = new HashMap<>();
		int idAlloc = 0;
		for (AbstractTable table : tables.values()) {
			StorageInterface storage = table.open(
					meta.getTable(table.getName()), logger);
			if (storage != null)
				storages.add(storage);
			TTable<?, ?> ttable = (TTable<?, ?>) table;
			String lockName = ttable.getLockName();
			Integer lockId = map.get(lockName);
			if (lockId == null)
				map.put(lockName, lockId = ++idAlloc);
			ttable.setLockId(lockId);
		}
		add(tableSys);
	}

	final void close() {
		storages.clear();
		tables.values().forEach(table -> table.close());
		if (null != logger) {
			logger.close();
			logger = null;
		}
	}

	final Lock flushReadLock() {
		return flushLock.readLock();
	}

	final boolean isFlushWriteLockHeldByCurrentThread() {
		return flushLock.isWriteLockedByCurrentThread();
	}

	final Lock flushWriteLock() {
		return flushLock.writeLock();
	}

	final List<StorageInterface> getStorages() {
		return storages;
	}

	final LoggerEngine getLogger() {
		return logger;
	}

}
